{library_calls/log_hlp.i}
{library_calls/libname.i}

bypass = false.

/* stdcall uses callee cleaning of the stack, so it is very sensitive to any case */
/* of mismatched argument sizes; when the 4GL trys using args that are not the same */
/* length as is expected by the functions called, the stack pointer will end up */
/* pointing to the wrong index (after the proper location when the passed args are */
/* larger than expected and before the proper location when the passed args are */
/* smaller than expected; the 4GL code that does the dispatching/calling can become */
/* unstable in this case, causing the process to abend; for this reason, we bypass */
/* such cases; this ONLY HAPPENS for INPUT parms because the other parm types use */
/* pointers (INPUT-OUTPUT or OUTPUT) or use registers for returning data and thus */
/* cannot destabilize the process */
if "{&callconv}" eq "stdcall" then
do:
   if "{&apitype}" eq "int64" and "{&rettype}" ne "int64" then
   do:
      bypass = true.
      run log-it (input "{&apiname} - parm type {&apitype} into actual library type {&rettype} BYPASSED").
   end.
   if "{&apitype}" ne "int64" and "{&rettype}" eq "int64" then
   do:
      bypass = true.
      run log-it (input "{&apiname} - parm type {&apitype} into actual library type {&rettype} BYPASSED").
   end.
end.

if not bypass then
do:
   run log-it (input "{&apiname} - parm type {&apitype} into actual library type {&rettype} start").
   
   is-small = ("{&vartype}" eq "integer").
   
   /* potential overflow case requires checking below */
   /* &vartype is the 4GL variable type to which we are assigning */
   /* &apitype is the 4GL shared library type name that is used to interpret the returned value */
   /* &rettype is the actual shared library function type being returned */
                                                
   do i = 1 to sz:
   
      if (is-small and (i-input[i] < -2147483648 or i-input[i] > 2147483647)) then
      do:
         /* pass a value (from the reversed array of integers copied out of the DLL) */
         {&varname} = little-input[i].
      end.
      else
      do:
         /* pass a value (from the reversed array of int64 copied out of the DLL) */
         {&varname} = i-input[i].
      end.
      
      /* deal with the INPUT overflow case */
      case "{&apitype}":
         when "byte" then
            do:
               if {&varname} < -128 or {&varname} > 255 then {&varname} = get-bits({&varname}, 1, 8).
            end.
         when "short" then
            do:
               if {&varname} < -32768 or {&varname} > 32767 then {&varname} = get-bits({&varname}, 1, 15).
            end.
         when "unsigned-short" then
            do:
               if {&varname} < 0 or {&varname} > 65535 then {&varname} = get-bits({&varname}, 1, 16).
            end.
         when "long" then
            do:
               if {&varname} < -2147483648 or {&varname} > 2147483647 then {&varname} = get-bits({&varname}, 1, 31).
            end.
         when "unsigned-long" then
            do:
               if {&varname} < 0 or {&varname} > 4294967295 then {&varname} = get-bits({&varname}, 1, 31).
            end.
         when "int64" then
            do:
               /* nothing to do, there can't be an overflow here */
            end.
         otherwise
            run log-err (input "Unknown apitype {&apitype} in {&apiname}.").
      end.
      
      i-tmp = {&varname}.
   
      err-flag = true.
      inner:
      do on error undo, leave inner:
         run {&apiname} (input {&varname}).
         err-flag = false.
      end.
      if err-flag then
         run log-err (input "Failure occured in {&apiname}() - apitype {&apitype} into {&vartype} variable from rettype {&rettype} for value (input value: " + string(i-tmp) + "; entry: " + string(i) + ").").
   end.
   
   /* prepare the input array and handle the INPUT overflow case */
   do i = 1 to extent(inum_array):
      if i <= sz then
         do:
            case "{&apitype}":
               when "byte" then
                  do:
                     if i-input[i] < -128 or i-input[i] > 255 then inum_array[i] = get-bits(i-input[i], 1, 8).
                     else inum_array[i] = i-input[i]. 
                  end.
               when "short" then
                  do:
                     if i-input[i] < -32768 or i-input[i] > 32767 then inum_array[i] = get-bits(i-input[i], 1, 15).
                     else inum_array[i] = i-input[i].
                  end.
               when "unsigned-short" then
                  do:
                     if i-input[i] < 0 or i-input[i] > 65535 then inum_array[i] = get-bits(i-input[i], 1, 16).
                     else inum_array[i] = i-input[i].
                  end.
               when "long" then
                  do:
                     if i-input[i] < -2147483648 or i-input[i] > 2147483647 then inum_array[i] = get-bits(i-input[i], 1, 31).
                     else inum_array[i] = i-input[i].
                  end.
               when "unsigned-long" then
                  do:
                     if i-input[i] < 0 or (is-small and i-input[i] > 2147483647) or (not is-small and i-input[i] > 4294967295) then inum_array[i] = get-bits(i-input[i], 1, 31).
                     else inum_array[i] = i-input[i].
                  end.
               when "int64" then
                  do:
                     if is-small and (i-input[i] < -2147483648 or i-input[i] > 2147483647) then inum_array[i] = get-bits(i-input[i], 1, 31).
                     else inum_array[i] = i-input[i].
                  end.
               otherwise
                  run log-err (input "Unknown apitype {&apitype} in {&apiname} during array processing.").
            end.
         end.
      else
         inum_array[i] = 0.
   end.
   
   run {&apiname}_array (input inum_array).
   
   procedure {&apiname} external "{&libname}" {&callconv}:
      define input parameter {&varname} as {&apitype}.
   end.
   
   procedure {&apiname}_array external "{&libname}" {&callconv}:
      define input parameter {&varname} as {&apitype}.
   end.
   
   run log-it (input "{&apiname} - parm type {&apitype} into actual library type {&rettype} end").

end.
