{library_calls/log_hlp.i}

run log-it (input "input_output_char_ptr_into_memptr_{&callconv} start").

{library_calls/libname.i}

def var txt as character init "bogus".
def var ptr as memptr.
def var ptr2ptr as memptr.
def var ptr-array as memptr extent 3.
def var bogus-ptr as memptr.
def var addr1 as int64.
def var addr2 as int64.
def var addr-other as int64.
def var sz as int.
def var i as int.

/* with input-output parms (as opposed to return parms), we must pre-allocate the buffers passed */
/* otherwise we get this: */
/* DLL procedure input_output_char_ptr_cdecl library_calls/input_output/input_output_parameter_char_ptr_into_memptr_cdecl.p using an uninitialised MEMPTR. (3233) */
run input_output_char_ptr_{&callconv} (input-output ptr) no-error.
if not error-status:error or not error-status:get-number(1) eq 3233 then run log-err (input "Expected an error passing an uninitialized memptr var for an input-output parameter (got '" + get-string(ptr, 1) + "').").

set-size(ptr) = 256.
set-size(ptr2ptr) = 8.

/* this will copy the data into the memptr buffer as a null-terminated string */
put-string(ptr, 1) = txt.
run input_output_char_ptr_{&callconv} (input-output ptr).
txt = get-string(ptr, 1).
if txt ne "This is sample text!" then run log-err (input "Unexpected text output via memptr ('" + txt + "').").

/* test "null" pointers; first one is explicitly sized 0, the second is uninitialized and the last as a real size (it isn't null) */
set-size(ptr-array[1]) = 0.
set-size(ptr-array[3]) = 256.
put-string(ptr-array[3], 1) = txt.
run input_output_char_ptr_{&callconv}_array (input-output ptr-array, input extent(ptr-array)).
if get-size(ptr-array[1]) ne 0 or get-pointer-value(ptr-array[1]) ne 0 then run log-err (input "Zero sized memptr was allocated by the DLL call.").
if get-size(ptr-array[2]) ne 0 or get-pointer-value(ptr-array[2]) ne 0 then run log-err (input "Uninitialized memptr was allocated by the DLL call.").
txt = get-string(ptr-array[3], 1).
if txt ne "2" then run log-err (input "Unexpected text output via memptr array element 3, expected 2 and got '" + txt + "'.").

/* passing any one of the array elements as unknown value causes an error (this is different from output parms): */
/* A variable or array element passed as an INPUT or INPUT-OUTPUT parameter to a DLL cannot contain the Unknown value. (12272) */
set-size(ptr-array[1]) = 256.
put-string(ptr-array[1], 1) = txt.
ptr-array[2] = ?.
set-size(ptr-array[3]) = 256.
put-string(ptr-array[3], 1) = txt.

/* strangely, NO-ERROR only suppresses the message here (and records the error) BUT the error is still raised! */
catcher1:
do on error undo, leave catcher1:
   run input_output_char_ptr_{&callconv}_array (input-output ptr-array, input extent(ptr-array)) no-error.
   run log-err (input "Does not reach here catcher1.").
end.
if not error-status:error or not error-status:get-number(1) eq 12272 then run log-err (input "Expected an error 12272 passing an unknown memptr var for 2nd input-output array element.").

/* passing all elements as unknown causes an error (this is different from output parms): */
/* A variable or array element passed as an INPUT or INPUT-OUTPUT parameter to a DLL cannot contain the Unknown value. (12272) */
do i = 1 to extent(ptr-array):
   ptr-array[i] = ?.
end.

/* strangely, NO-ERROR only suppresses the message here (and records the error) BUT the error is still raised! */
catcher2:
do on error undo, leave catcher2:
   run input_output_char_ptr_{&callconv}_array (input-output ptr-array, input extent(ptr-array)) no-error.
   run log-err (input "Does not reach here catcher2.").
end.
if not error-status:error or not error-status:get-number(1) eq 12272 then run log-err (input "Expected an error 12272 passing an unknown memptr var for all input-output array elements.").

/* normal array case */
do i = 1 to extent(ptr-array):
   set-size(ptr-array[i]) = 0.
   set-size(ptr-array[i]) = 256.
   put-string(ptr-array[i], 1) = txt + " " + string(i).
end.

run input_output_char_ptr_{&callconv}_array (input-output ptr-array, input extent(ptr-array)).
do i = 1 to extent(ptr-array):
   txt = get-string(ptr-array[i], 1).
   if txt ne string(i - 1) then run log-err (input "Unexpected text output via memptr array, expected " + string(i - 1) + " and got '" + txt + "'.").
end.

run ptr_size (output sz).

set-size(bogus-ptr) = 16.
put-string(bogus-ptr, 1) = "Trash here!".
addr-other = get-pointer-value(bogus-ptr).

if sz = 4 then
   put-long(ptr2ptr, 1) = addr-other.
else
   put-int64(ptr2ptr, 1) = addr-other.

/* this will set the memptr contents to be a pointer to the DLL memory that contains a */
/* null-terminated string */
run input_output_char_ptr_ptr_{&callconv} (input-output ptr2ptr).

set-size(bogus-ptr) = 0.

if sz = 4 then
   addr2 = get-long(ptr2ptr, 1).
else
   addr2 = get-int64(ptr2ptr, 1).

set-size(ptr) = 0.
set-size(ptr2ptr) = 0.

/* this will return an allocated buffer in the DLL */
run return_char_ptr_{&callconv} (output ptr).
addr1 = get-pointer-value(ptr).

if addr1 ne addr2 then run log-err (input "Expected address " + string(addr1) + " and got " + string(addr2) + ".").

set-pointer-value(ptr) = addr2.
txt = get-string(ptr, 1).
if txt ne "This is sample text!" then run log-err (input "Unexpected text output (2) via memptr ('" + txt + "').").

/* no de-allocation of the memptrs is needed since we don't "own" the memory, the DLL does */

procedure ptr_size external "{&libname}" cdecl persistent:
   def return parameter sz as long.
end.

procedure input_output_char_ptr_{&callconv} external "{&libname}" {&callconv} persistent:
   def input-output parameter ptr as memptr.
end.

procedure input_output_char_ptr_ptr_{&callconv} external "{&libname}" {&callconv} persistent:
   def input-output parameter ptr as memptr.
end.

procedure input_output_char_ptr_{&callconv}_array external "{&libname}" {&callconv} persistent:
   def input-output parameter ptr as memptr.
   def input parameter i as long.
end.

procedure return_char_ptr_{&callconv} external "{&libname}" {&callconv} persistent:
   def return parameter ptr as memptr.
end.

run log-it (input "input_output_char_ptr_into_memptr_{&callconv} end").
