{library_calls/log_hlp.i}

run log-it (input "{&apiname} reading {&apitype} error processing start").

{library_calls/libname.i}

/* This call will cause numerous messages to be displayed if run without NO-ERROR. */
/* The messages are not treated as "errors" (even without NO-ERROR), so the ERROR */
/* condition is not raised and when NO-ERROR is used there is no update of the */
/* ERROR-STATUS list of errors (well, the list isn't updated w/o NO-ERROR either, */
/* but that is normal). */
/* There are 4 messages: ** Insufficient space for float to decimal conversion. (86) */
/* The problem only occurs calling input_output_double_cdecl() when specifying the */
/* parameter type as float. */
/* By changing the output values that the DLL writes back it was found that these */
/* "complaints" are only caused by some combination of the data being written back, */
/* possibly in combination with the data that was input as well.  The important points */
/* here: */
/* 1. If safe values were written back (e.g. 0) then there would not be any complaints. */
/*    There are no errors on INPUT. */
/* 2. When we write only a 1 element array (actually it is 2 elements but the DLL is told */
/*    that it is only 1 element long), there are many fewer complaints than occur in this */
/*    same situation in the single big array call in input_output_float_test.i. */
do i = 1 to sz:

   /* pass a value (from the reversed array of decimals copied out of the DLL) */
   dnum_array[1] = d-input[i].

   err-flag = true.
   inner:
   do on error undo, leave inner:
      run {&apiname}_limit_array (input-output dnum_array, input 1, input i) no-error.
      err-flag = false.
   end.
   if err-flag then
   do:
      err-list = "".
      do i = 1 to error-status:num-messages:
         err-list = err-list + string(error-status:get-number(i)) + " ".
      end.
      run log-err (input "Failure occured in {&apiname}() - {&apitype} into decimal for entry: " + string(i) + "; input = " + string(d-input[i]) + "; error list = " + err-list).
   end.
end.

procedure {&apiname}_limit_array external "{&libname}" {&callconv}:
   define input-output parameter d as {&apitype}.
   define input parameter len as long.
   define input parameter idx as long.
end.

run log-it (input "{&apiname} reading {&apitype} error processing end").
