{library_calls/log_hlp.i}

run log-it (input "{&apiname} - parm type {&apitype} into actual library type {&rettype} start").

{library_calls/libname.i}

/* allocating and deallocating this in each included section results in the */
/* memory being cleared before each test, which is important */
set-size(results) = sz * 8.

is-small = ("{&vartype}" eq "integer").

/* potential overflow case requires checking below */
/* &vartype is the 4GL variable type to which we are assigning */
/* &apitype is the 4GL shared library type name that is used to interpret the returned value */
/* &rettype is the actual shared library function type being returned */
s-overflow = is-small and ("{&apitype}" eq "int64") and ("{&rettype}" eq "int64").
n-overflow = is-small and ("{&apitype}" eq "int64") and ("{&rettype}" eq "int32" or "{&rettype}" eq "uint32").
u-overflow = is-small and ("{&apitype}" eq "unsigned-long") and ("{&rettype}" eq "int32" or "{&rettype}" eq "uint32" or "{&rettype}" eq "int64").
                                             
do i = 1 to sz:

   if (is-small and (i-input[i] < -2147483648 or i-input[i] > 2147483647)) then
   do:
      /* pass a value (from the reversed array of integers copied out of the DLL) */
      {&varname} = little-input[i].
   end.
   else
   do:
      /* pass a value (from the reversed array of int64 copied out of the DLL) */
      {&varname} = i-input[i].
   end.
   
   /* deal with the INPUT overflow case */
   case "{&apitype}":
      when "byte" then
         do:
            if {&varname} < -128 or {&varname} > 255 then {&varname} = get-bits({&varname}, 1, 8).
         end.
      when "short" then
         do:
            if {&varname} < -32768 or {&varname} > 32767 then {&varname} = get-bits({&varname}, 1, 15).
         end.
      when "unsigned-short" then
         do:
            if {&varname} < 0 or {&varname} > 65535 then {&varname} = get-bits({&varname}, 1, 16).
         end.
      when "long" then
         do:
            if {&varname} < -2147483648 or {&varname} > 2147483647 then {&varname} = get-bits({&varname}, 1, 31).
         end.
      when "unsigned-long" then
         do:
            if {&varname} < 0 or {&varname} > 4294967295 then {&varname} = get-bits({&varname}, 1, 31).
         end.
      when "int64" then
         do:
            /* nothing to do, there can't be an overflow here */
         end.
      otherwise
         run log-err (input "Unknown apitype {&apitype} in {&apiname}.").
   end.
   
   i-tmp = {&varname}.

   /* make the call being tested */
   if (s-overflow and (ilist[i] < -2147483648 or ilist[i] > 2147483647)) or
      (u-overflow and get-bits(ilist[i], 32, 1) eq 1) or                       /* <-- this check had to be different for the INPUT-OUTPUT case */
      (n-overflow and {&varname} >= 0 and get-bits(ilist[i], 32, 1) eq 1) or   /* <-- this check had to be different for the INPUT-OUTPUT case */
      (n-overflow and {&varname} < 0 and funky-int-mashup-is-out-of-bounds(i)) /* <-- this extra check is needed for the INPUT-OUTPUT case */
   then
   do:
      /* deal with the OUTPUT overflow case */
      run {&apiname} (input-output {&varname}) no-error.
      if not error-status:error or not error-status:get-number(1) eq 15747 then run log-err (input "Unexpected result in {&apiname}() - {&apitype} into {&vartype} for value (input value: " + string(i-tmp) + "; ori: " + string(ilist[i]) + "; entry: " + string(i) + "; read: " + string({&varname}) + ").").
   end.
   else
   do:
      err-flag = true.
      inner:
      do on error undo, leave inner:
         run {&apiname} (input-output {&varname}).
         err-flag = false.
      end.
      if err-flag then
         run log-err (input "Failure occured in {&apiname}() - apitype {&apitype} into {&vartype} variable from rettype {&rettype} for value (input value: " + string(i-tmp) + "; ori: " + string(ilist[i]) + "; entry: " + string(i) + ").").
   end.
   
   /* save results into buffer */
   idx = ((i - 1) * 8) + 1.
   put-int64(results, idx) = {&varname}.
end.

copy-lob results to file "{&apiname}_{&apitype}_into_{&vartype}.out".

/* prepare the input array and handle the INPUT overflow case */
do i = 1 to extent(inum_array):
   if i <= sz then
      do:
         case "{&apitype}":
            when "byte" then
               do:
                  if i-input[i] < -128 or i-input[i] > 255 then inum_array[i] = get-bits(i-input[i], 1, 8).
                  else inum_array[i] = i-input[i]. 
               end.
            when "short" then
               do:
                  if i-input[i] < -32768 or i-input[i] > 32767 then inum_array[i] = get-bits(i-input[i], 1, 15).
                  else inum_array[i] = i-input[i].
               end.
            when "unsigned-short" then
               do:
                  if i-input[i] < 0 or i-input[i] > 65535 then inum_array[i] = get-bits(i-input[i], 1, 16).
                  else inum_array[i] = i-input[i].
               end.
            when "long" then
               do:
                  if i-input[i] < -2147483648 or i-input[i] > 2147483647 then inum_array[i] = get-bits(i-input[i], 1, 31).
                  else inum_array[i] = i-input[i].
               end.
            when "unsigned-long" then
               do:
                  if i-input[i] < 0 or (is-small and i-input[i] > 2147483647) or (not is-small and i-input[i] > 4294967295) then inum_array[i] = get-bits(i-input[i], 1, 31).
                  else inum_array[i] = i-input[i].
               end.
            when "int64" then
               do:
                  if is-small and (i-input[i] < -2147483648 or i-input[i] > 2147483647) then inum_array[i] = get-bits(i-input[i], 1, 31).
                  else inum_array[i] = i-input[i].
               end.
            otherwise
               run log-err (input "Unknown apitype {&apitype} in {&apiname} during array processing.").
         end.
      end.
   else
      inum_array[i] = 0.
end.

if ("{&vartype}" eq "integer") and ("{&apitype}" eq "int64" or "{&apitype}" eq "unsigned-long") then
do:
   /* these cases always generate the error 15747 (no matter what rettype is used and then halt the array processing */
   /* Value xx too large to fit in INTEGER. (15747) */
   /* the array copying seems to be an "internal copying loop" that tries to instantiate integer instances but */
   /* which can be aborted on encountering the first failure, since only 1 error is recorded in the error-status */
   /* handle AND there is very little data copied into the array; this is different from the decimal "failures" */
   /* which just show some messages but don't treat them as error conditions; the interesting thing is that the NO-ERROR */
   /* does suppress the ERROR condition from being raised here but it doesn't stop the internal copy loop from */
   /* aborting */
   run {&apiname}_array (input-output inum_array) no-error.
   if not error-status:error or error-status:get-number(1) ne 15747 then run log-err (input "Expected error 15747 on output array parameter copying from {&apiname}() using {&apitype} parm type into var type {&vartype}.  ERROR = " + string(error-status:error) + " and msg 1 is " + string(error-status:get-number(1))).
end.
else
do:
   run {&apiname}_array (input-output inum_array).
end.

set-size(results) = 0.
set-size(results) = extent(inum_array) * 8.

/* save results into buffer */
do i = 1 to extent(inum_array):
   idx = ((i - 1) * 8) + 1.
   put-int64(results, idx) = inum_array[i].
end.

copy-lob results to file "{&apiname}_{&apitype}_array_into_{&vartype}.out".

set-size(results) = 0.

procedure {&apiname} external "{&libname}" {&callconv}:
   define input-output parameter {&varname} as {&apitype}.
end.

procedure {&apiname}_array external "{&libname}" {&callconv}:
   define input-output parameter {&varname} as {&apitype}.
end.

run log-it (input "{&apiname} - parm type {&apitype} into actual library type {&rettype} end").
