{library_calls/log_hlp.i}
{library_calls/libname.i}

def var i as int.
def var rc as int.

run log-it (input "name_decoration start").

/* does the 4GL have a lookup algorithm that uses an _ prefixed function name if the */
/* undecorated function name cannot be found? NO, it DOESN'T! */
run underscore_prefixed_function_name (output rc) no-error.
if not error-status:error or
   not error-status:get-number(1) eq 3260
   then run log-err (input "Expected error 3260 for mismatched function name (1). Got '" + error-status:get-message(1) + "'.").
   
rc = 0.
run _underscore_prefixed_function_name (output rc).
if rc <> 1 then run log-err (input "Expected rc = 1 but got " + string(rc) + " (1).").
   
run ptr_size (output i).
if i ne 4 and i ne 8 then run log-err (input "Expected a valid pointer size to be returned.  Got '" + string(i) + "'.").

/* windows 32-bit specific case (STDCALL decorations) */
if opsys eq "win32" and i = 4 then
do:

   /* Does the 4GL have a lookup algorithm that uses an @ num suffixed function name if the */
   /* undecorated function name cannot be found? It KIND OF does. */
   
   /* this has no underscore prefix, but does have the @N suffix, Progress does not find it */
   rc = 0.
   run function_name_with_at_suffix (input 14, output rc) no-error.
   if not error-status:error or
      not error-status:get-number(1) eq 3260
      then run log-err (input "Expected error 3260 for mismatched function name (2). Got '" + error-status:get-message(1) + "'.").
   
   /* interesting result: Progress does try to lookup stdcall funcs by decorating the name */
   /* with BOTH an underscore prefix AND an @X suffix that they must calculate from the parms */
   /* the real function in this case has this name: _underscore_prefixed_function_name_with_at_suffix@4 */
   rc = 0.
   run underscore_prefixed_function_name_with_at_suffix (input 14, output rc) no-error.
   if rc <> 1 then run log-err (input "Expected rc = 1 but got " + string(rc) + " (2).").
      
   /* Progress does not try to just add an @N suffix to a function name that already has an underscore */
   /* (but it probably just adds an extra underscore and checks, which would fail the same way). */ 
   /* the real function in this case has this name: _underscore_prefixed_function_name_with_at_suffix@4 */
   run _underscore_prefixed_function_name_with_at_suffix (input 14, output rc) no-error.
   if not error-status:error or
      not error-status:get-number(1) eq 3260
      then run log-err (input "Expected error 3260 for mismatched function name (3). Got '" + error-status:get-message(1) + "'.").
      
   /* When Progress doesn't find this name, it adds both an extra underscore prefix and the @N */
   /* suffix and then tries the lookup again. This time it finds it! */
   /* the real function in this case has this name: __double_underscore_prefixed_function_name_with_at_suffix@4 */
   rc = 0.
   run _double_underscore_prefixed_function_name_with_at_suffix (input 14, output rc) no-error.
   if rc <> 1 then run log-err (input "Expected rc = 1 but got " + string(rc) + " (3).").
      
   rc = 0.
   run function_name_with_at_suffix@4 (input 14, output rc).
   if rc <> 1 then run log-err (input "Expected rc = 1 but got " + string(rc) + " (4).").
   
   rc = 0.
   run _underscore_prefixed_function_name_with_at_suffix@4 (input 14, output rc).
   if rc <> 1 then run log-err (input "Expected rc = 1 but got " + string(rc) + " (5).").
      
   procedure function_name_with_at_suffix external "{&libname2}" stdcall:
      def input parameter sz as long.
      def return parameter i as long.
   end.
   
   procedure underscore_prefixed_function_name_with_at_suffix external "{&libname2}" stdcall:
      def input parameter sz as long.
      def return parameter i as long.
   end.
   
   procedure _underscore_prefixed_function_name_with_at_suffix external "{&libname2}" stdcall:
      def input parameter sz as long.
      def return parameter i as long.
   end.
   
   procedure _double_underscore_prefixed_function_name_with_at_suffix external "{&libname2}" stdcall:
      def input parameter sz as long.
      def return parameter i as long.
   end.
   
   procedure function_name_with_at_suffix@4 external "{&libname2}" stdcall:
      def input parameter sz as long.
      def return parameter i as long.
   end.
   
   procedure _underscore_prefixed_function_name_with_at_suffix@4 external "{&libname2}" stdcall:
      def input parameter sz as long.
      def return parameter i as long.
   end.
end.

run log-it (input "name_decoration end").

/* this function name doesn't exist */
procedure underscore_prefixed_function_name external "{&libname}" cdecl:
   def return parameter i as long.
end.

/* this function name does exist */
procedure _underscore_prefixed_function_name external "{&libname}" cdecl:
   def return parameter i as long.
end.

procedure ptr_size external "{&libname}" cdecl:
   def return parameter sz as long.
end.
