{library_calls/log_hlp.i}

run log-it (input "output_char_ptr_into_memptr_{&callconv} start").

{library_calls/libname.i}

def var txt as character init "bogus".
def var ptr as memptr.
def var ptr2ptr as memptr.
def var ptr-array as memptr extent 3.
def var addr1 as int64.
def var addr2 as int64.
def var sz as int.
def var i as int.

/* with output parms (as opposed to return parms), we must pre-allocate the buffers passed */
/* otherwise we get this: */
/* DLL procedure output_char_ptr_cdecl library_calls/output/output_parameter_char_ptr_into_memptr_cdecl.p using an uninitialised MEMPTR. (3233) */
run output_char_ptr_{&callconv} (output ptr) no-error.
if not error-status:error or not error-status:get-number(1) eq 3233 then run log-err (input "Expected an error passing an uninitialized memptr var for an output parameter (got '" + get-string(ptr, 1) + "').").

set-size(ptr) = 256.
set-size(ptr2ptr) = 8.

/* this will copy the data into the memptr buffer as a null-terminated string */
run output_char_ptr_{&callconv} (output ptr).
txt = get-string(ptr, 1).
if txt ne "This is sample text!" then run log-err (input "Unexpected text output via memptr ('" + txt + "').").

/* test "null" pointers; first one is explicitly sized 0, the second is uninitialized and the last as a real size (it isn't null) */
set-size(ptr-array[1]) = 0.
set-size(ptr-array[3]) = 256.
run output_char_ptr_{&callconv}_array (output ptr-array, input extent(ptr-array)).
if get-size(ptr-array[1]) ne 0 or get-pointer-value(ptr-array[1]) ne 0 then run log-err (input "Zero sized memptr was allocated by the DLL call.").
if get-size(ptr-array[2]) ne 0 or get-pointer-value(ptr-array[2]) ne 0 then run log-err (input "Uninitialized memptr was allocated by the DLL call.").
txt = get-string(ptr-array[3], 1).
if txt ne "2" then run log-err (input "Unexpected text output via memptr array element 3, expected 2 and got '" + txt + "'.").

/* passing any one of the array elements as unknown value causes that element to be passes as null */
set-size(ptr-array[1]) = 256.
ptr-array[2] = ?.
set-size(ptr-array[3]) = 256.
run output_char_ptr_{&callconv}_array (output ptr-array, input extent(ptr-array)).

txt = get-string(ptr-array[1], 1).
if txt ne "0" then run log-err (input "Unexpected text output via memptr array element 1 when there was an array element with unknown value, expected '0' and got '" + txt + "'.").
if ptr-array[2] ne ? then run log-err (input "Unexpected text output via memptr array, expected ? and got '" + get-string(ptr-array[i], 1) + "'.").
txt = get-string(ptr-array[3], 1).
if txt ne "2" then run log-err (input "Unexpected text output via memptr array element 3 when there was an array element with unknown value, expected '2' and got '" + txt + "'.").

/* passing all elements as unknown still calls the API, just with all 3 pointers set to NULL */
do i = 1 to extent(ptr-array):
   ptr-array[i] = ?.
end.

run output_char_ptr_{&callconv}_array (output ptr-array, input extent(ptr-array)).
do i = 1 to extent(ptr-array):
   if ptr-array[i] ne ? then run log-err (input "Unexpected text output via all-unknown memptr array, expected ? and got '" + get-string(ptr-array[i], 1) + "'.").
end.

/* normal array case */
do i = 1 to extent(ptr-array):
   set-size(ptr-array[i]) = 0.
   set-size(ptr-array[i]) = 256.
end.

run output_char_ptr_{&callconv}_array (output ptr-array, input extent(ptr-array)).
do i = 1 to extent(ptr-array):
   txt = get-string(ptr-array[i], 1).
   if txt ne string(i - 1) then run log-err (input "Unexpected text output via memptr array, expected " + string(i - 1) + " and got '" + txt + "'.").
end.

/* this will set the memptr contents to be a pointer to the DLL memory that contains a */
/* null-terminated string */
run output_char_ptr_ptr_{&callconv} (output ptr2ptr).

run ptr_size (output sz).

if sz = 4 then
   addr2 = get-long(ptr2ptr, 1).
else
   addr2 = get-int64(ptr2ptr, 1).

set-size(ptr) = 0.
set-size(ptr2ptr) = 0.

/* this will return an allocated buffer in the DLL */
run return_char_ptr_{&callconv} (output ptr).
addr1 = get-pointer-value(ptr).

if addr1 ne addr2 then run log-err (input "Expected address " + string(addr1) + " and got " + string(addr2) + ".").

set-pointer-value(ptr) = addr2.
txt = get-string(ptr, 1).
if txt ne "This is sample text!" then run log-err (input "Unexpected text output (2) via memptr ('" + txt + "').").

/* no de-allocation of the memptrs is needed since we don't "own" the memory, the DLL does */

procedure ptr_size external "{&libname}" cdecl persistent:
   def return parameter sz as long.
end.

procedure output_char_ptr_{&callconv} external "{&libname}" {&callconv} persistent:
   def output parameter ptr as memptr.
end.

procedure output_char_ptr_ptr_{&callconv} external "{&libname}" {&callconv} persistent:
   def output parameter ptr as memptr.
end.

procedure output_char_ptr_{&callconv}_array external "{&libname}" {&callconv} persistent:
   def output parameter ptr as memptr.
   def input parameter i as long.
end.

procedure return_char_ptr_{&callconv} external "{&libname}" {&callconv} persistent:
   def return parameter ptr as memptr.
end.

run log-it (input "output_char_ptr_into_memptr_{&callconv} end").
