{library_calls/log_hlp.i}

run log-it (input "output_parameter_char_actually_works start").

{library_calls/libname.i}

def var i as int.
def var cmp-txt as char.
def var err-flag as logical.

/* these must be big enough to hold the changes otherwise you might see an error like */
/* Your LONGCHAR buffer has been overrun.  Memory has been corrupted. (14528) */
def var txt as character init "012345678901234567890123456789012345678901234567890123456789".
def var ltxt as longchar init "012345678901234567890123456789012345678901234567890123456789".

def var txt-array as character extent 3.
def var ltxt-array as longchar extent 3.

txt-array[1]  = "abcdefghijk".
txt-array[2]  = "abcdefghijk".
txt-array[3]  = "abcdefghijk".
ltxt-array[1] = "abcdefghijk".
ltxt-array[2] = "abcdefghijk".
ltxt-array[3] = "abcdefghijk".

/* the 4GL docs state that the ABL does not allow you to use output with character or longchar; */
/* BUT it turns out that it works fine on v10.2B */

run output_char_ptr_cdecl (output txt).
if txt ne "This is sample text!" then run log-err (input "Expected no change to variable but txt = '" + txt + "'").
run output_char_ptr_cdecl (output ltxt).
if ltxt ne "This is sample text!" then run log-err (input "Expected no change to variable but ltxt = '" + ltxt + "'").

run output_char_ptr_cdecl_array (output txt-array, input extent(txt-array)).

do i = 1 to extent(txt-array):
   cmp-txt = string((i - 1)).
   if txt-array[i] ne cmp-txt then run log-err (input "Expected " + cmp-txt + " but instead found txt-array[" + string(i) + "] = '" + txt-array[i] + "'").
end.
   
run output_char_ptr_cdecl_array (output ltxt-array, input extent(ltxt-array)).

do i = 1 to extent(ltxt-array):
   cmp-txt = string((i - 1)).
   if ltxt-array[i] ne cmp-txt then run log-err (input "Expected " + cmp-txt + " but instead found ltxt-array[" + string(i) + "] = '" + ltxt-array[i] + "'").
end.

/* empty string in array elements will work (the result is also an empty string, but the function is actually invoked with real pointers, but no contents are copied back!) */
do i = 1 to extent(txt-array):
   txt-array[i] = "".
end.

run output_char_ptr_cdecl_array (output txt-array, input extent(txt-array)).

do i = 1 to extent(txt-array):
   if txt-array[i] ne "" then run log-err (input "Expected empty string but instead found txt-array[" + string(i) + "] = '" + txt-array[i] + "'").
end.

/* empty string in longchar array elements causes an error */
/* A CHARACTER or LONGCHAR variable passed as an OUTPUT parameter to a DLL cannot be the Unknown value or be uninitialised. (12450) */
do i = 1 to extent(ltxt-array):
   ltxt-array[i] = "".
end.

/* on Windows, there is an internal 4GL segmentation fault (in one of the Progress DLLs) when */
/* this next call is processed, the entire process dies and NOT in the called DLL, so we bypass */
/* this on Windows */
if opsys ne "win32" then
do:
   run output_char_ptr_cdecl_array (output ltxt-array, input extent(ltxt-array)) no-error.
   if not error-status:error or not error-status:get-number(1) eq 12450 then run log-err (input "Expected an error passing an unknown longchar element in an output array parameter (got element 1 as '" + string(ltxt-array[1]) + "').").
end.
   
/* unknown value in array elements raises an error condition */
/* A CHARACTER or LONGCHAR variable passed as an OUTPUT parameter to a DLL cannot be the Unknown value or be uninitialised. (12450) */
do i = 1 to extent(txt-array):
   txt-array[i] = ?.
end.

/* this silently generates an error when no-error is specified! */
err-flag = true.
do on error undo, leave:
   run output_char_ptr_cdecl_array (output txt-array, input extent(txt-array)) no-error.
   err-flag = false.
end.
if not err-flag or not error-status:error or not error-status:get-number(1) eq 12450 then run log-err (input "Expected an error passing an unknown character element in an output array parameter (got element 1 as '" + txt-array[1] + "').").

/* unknown value in longchar array elements raises an error condition */
/* A CHARACTER or LONGCHAR variable passed as an OUTPUT parameter to a DLL cannot be the Unknown value or be uninitialised. (12450) */
do i = 1 to extent(ltxt-array):
   ltxt-array[i] = ?.
end.

/* this silently generates an error when no-error is specified! */
err-flag = true.
do on error undo, leave:
   run output_char_ptr_cdecl_array (output ltxt-array, input extent(ltxt-array)) no-error.
err-flag = false.
end.
if not err-flag or not error-status:error or not error-status:get-number(1) eq 12450 then run log-err (input "Expected an error passing an unknown longchar element in an output array parameter (got element 1 as '" + string(ltxt-array[1]) + "').").
   
procedure output_char_ptr_cdecl external "{&libname}" cdecl persistent:
   /* using this form makes no difference: def output parameter ptr as handle to character. */
   def output parameter ptr as character.
end.

procedure output_char_ptr_cdecl_array external "{&libname}" cdecl persistent:
   def output parameter ptr as character.
   def input parameter len as long.
end.

run log-it (input "output_parameter_char_actually_works end").
